/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
//@line 44 "/export/home/ffslavex/mozilla-1.9.2-solaris-x86-nightly/build/toolkit/mozapps/update/src/nsUpdateService.js.in"
*/

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

const PREF_APP_UPDATE_LASTUPDATETIME_FMT  = "app.update.lastUpdateTime.%ID%";
const PREF_APP_UPDATE_TIMER               = "app.update.timer";
const CATEGORY_UPDATE_TIMER               = "update-timer";

//@line 136 "/export/home/ffslavex/mozilla-1.9.2-solaris-x86-nightly/build/toolkit/mozapps/update/src/nsUpdateService.js.in"

__defineGetter__("gPref", function () {
  delete this.gPref;
  return this.gPref = Cc["@mozilla.org/preferences-service;1"].
                      getService(Ci.nsIPrefBranch2);
});

/**
 * Gets a preference value, handling the case where there is no default.
 * @param   func
 *          The name of the preference function to call, on nsIPrefBranch
 * @param   preference
 *          The name of the preference
 * @param   defaultValue
 *          The default value to return in the event the preference has
 *          no setting
 * @returns The value of the preference, or undefined if there was no
 *          user or default value.
 */
function getPref(func, preference, defaultValue) {
  try {
    return gPref[func](preference);
  }
  catch (e) {
  }
  return defaultValue;
}

__defineGetter__("gLogEnabled", function () {
  delete this.gLogEnabled;
  this.gLogEnabled = { };
  try {
    let ps = Cc["@mozilla.org/preferences-service;1"].
             getService(Ci.nsIPrefService);
    let logBranch = ps.getBranch(PREF_APP_UPDATE_LOG_BRANCH);
    let modules = logBranch.getChildList("", { value: 0 });
    for (let i = 0; i < modules.length; ++i) {
      if (logBranch.prefHasUserValue(modules[i]))
        this.gLogEnabled[modules[i]] = logBranch.getBoolPref(modules[i]);
    }
  }
  catch (e) {
  }
  return this.gLogEnabled;
});

__defineGetter__("gConsole", function () {
  delete this.gConsole;
  return this.gConsole = Cc["@mozilla.org/consoleservice;1"].
                         getService(Ci.nsIConsoleService);
});

/**
 * Logs a string to the error console.
 * @param   string
 *          The string to write to the error console..
 */
function LOG(module, string) {
  if (module in gLogEnabled || "all" in gLogEnabled) {
    dump("*** AUS:SVC " + module + ":" + string + "\n");
    gConsole.logStringMessage("AUS:SVC " + module + ":" + string);
  }
}

function getObserverService()
{
  return Cc["@mozilla.org/observer-service;1"].
         getService(Ci.nsIObserverService);
}

//@line 3127 "/export/home/ffslavex/mozilla-1.9.2-solaris-x86-nightly/build/toolkit/mozapps/update/src/nsUpdateService.js.in"

/**
 * A manager for timers. Manages timers that fire over long periods of time
 * (e.g. days, weeks, months).
 * @constructor
 */
function TimerManager() {
  let os = getObserverService();
  os.addObserver(this, "xpcom-shutdown", false);
}
TimerManager.prototype = {
  /**
   * The Checker Timer
   */
  _timer: null,

  /**
   * The Checker Timer interval as specified by the app.update.timer pref. If
   * the app.update.timer pref doesn't exist this will default to 600000.
   */
   _timerInterval: null,

  /**
   * The set of registered timers.
   */
  _timers: { },

  /**
   * The amount to fudge the lastUpdateTime where fudge is a random increment of
   * the update check interval (e.g. some random slice of 10 minutes). When the
   * time comes to notify a timer or a timer is first registered the timer is
   * offset by this amount to lessen the number of timers firing at the same
   * time. this._timerInterval is in milliseconds, whereas the lastUpdateTime is
   * in seconds so this._timerInterval is divided by 1000.
   */
  get _fudge() { return Math.round(Math.random() * this._timerInterval / 1000); },

  /**
   * See nsIObserver.idl
   */
  observe: function TM_observe(aSubject, aTopic, aData) {
    switch (aTopic) {
    case "profile-after-change":
      this._start();
      break;
    case "xpcom-shutdown":
      let os = getObserverService();
      os.removeObserver(this, "xpcom-shutdown");

      // Release everything we hold onto.
      if (this._timer) {
        this._timer.cancel();
        this._timer = null;
      }
      for (var timerID in this._timers)
        delete this._timers[timerID];
      this._timers = null;
      break;
    }
  },

  _start: function TM__start() {
    this._timerInterval = getPref("getIntPref", "app.update.timer", 600000);
    this._timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    this._timer.initWithCallback(this, this._timerInterval,
                                 Ci.nsITimer.TYPE_REPEATING_SLACK);
  },
  /**
   * Called when the checking timer fires.
   * @param   timer
   *          The checking timer that fired.
   */
  notify: function TM_notify(timer) {
    var prefLastUpdate;
    var lastUpdateTime;
    var now = Math.round(Date.now() / 1000);
    var catMan = Cc["@mozilla.org/categorymanager;1"].
                 getService(Ci.nsICategoryManager);
    var entries = catMan.enumerateCategory(CATEGORY_UPDATE_TIMER);
    while (entries.hasMoreElements()) {
      let entry = entries.getNext().QueryInterface(Ci.nsISupportsCString).data;
      let value = catMan.getCategoryEntry(CATEGORY_UPDATE_TIMER, entry);
      let [cid, method, timerID, prefInterval, defaultInterval] = value.split(",");
      defaultInterval = parseInt(defaultInterval);
      // cid and method are validated below when calling notify.
      if (!timerID || !defaultInterval || isNaN(defaultInterval)) {
        LOG("TimerManager", "notify - update-timer category registered" +
            (cid ? " for " + cid : "") + " without required parameters - " +
             "skipping");
        continue;
      }

      let interval = getPref("getIntPref", prefInterval, defaultInterval);
      prefLastUpdate = PREF_APP_UPDATE_LASTUPDATETIME_FMT.replace(/%ID%/,
                                                                      timerID);
      if (gPref.prefHasUserValue(prefLastUpdate)) {
        lastUpdateTime = gPref.getIntPref(prefLastUpdate);
      }
      else {
        lastUpdateTime = now + this._fudge;
        gPref.setIntPref(prefLastUpdate, lastUpdateTime);
        continue;
      }

      if ((now - lastUpdateTime) > interval) {
        try {
          Components.classes[cid][method](Ci.nsITimerCallback).notify(timer);
          LOG("TimerManager", "notify - notified " + cid);
        }
        catch (e) {
          LOG("TimerManager", "notify - error notifying component id: " +
              cid + " ,error: " + e);
        }
        lastUpdateTime = now + this._fudge;
        gPref.setIntPref(prefLastUpdate, lastUpdateTime);
      }
    }

    for (var timerID in this._timers) {
      var timerData = this._timers[timerID];

      if ((now - timerData.lastUpdateTime) > timerData.interval) {
        if (timerData.callback instanceof Ci.nsITimerCallback) {
          try {
            timerData.callback.notify(timer);
            LOG("TimerManager", "notify - notified timerID: " + timerID);
          }
          catch (e) {
            LOG("TimerManager", "notify - error notifying timerID: " + timerID +
                ", error: " + e);
          }
        }
        else {
          LOG("TimerManager", "notify - timerID: " + timerID + " doesn't " +
              "implement nsITimerCallback - skipping");
        }
        lastUpdateTime = now + this._fudge;
        timerData.lastUpdateTime = lastUpdateTime;
        prefLastUpdate = PREF_APP_UPDATE_LASTUPDATETIME_FMT.replace(/%ID%/, timerID);
        gPref.setIntPref(prefLastUpdate, lastUpdateTime);
      }
    }
  },

  /**
   * See nsIUpdateService.idl
   */
  registerTimer: function TM_registerTimer(id, callback, interval) {
    LOG("TimerManager", "registerTimer - id: " + id);
    var prefLastUpdate = PREF_APP_UPDATE_LASTUPDATETIME_FMT.replace(/%ID%/, id);
    var lastUpdateTime;
    if (gPref.prefHasUserValue(prefLastUpdate)) {
      lastUpdateTime = gPref.getIntPref(prefLastUpdate);
    } else {
      lastUpdateTime = Math.round(Date.now() / 1000) + this._fudge;
      gPref.setIntPref(prefLastUpdate, lastUpdateTime);
    }
    this._timers[id] = { callback       : callback,
                         interval       : interval,
                         lastUpdateTime : lastUpdateTime };
  },

  classDescription: "Timer Manager",
  contractID: "@mozilla.org/updates/timer-manager;1",
  classID: Components.ID("{B322A5C0-A419-484E-96BA-D7182163899F}"),
  _xpcom_categories: [{ category: "profile-after-change" }],
  QueryInterface: XPCOMUtils.generateQI([Ci.nsIUpdateTimerManager,
                                         Ci.nsITimerCallback,
                                         Ci.nsIObserver])
};

//@line 3302 "/export/home/ffslavex/mozilla-1.9.2-solaris-x86-nightly/build/toolkit/mozapps/update/src/nsUpdateService.js.in"
function NSGetModule(compMgr, fileSpec)
  XPCOMUtils.generateModule([TimerManager]);
//@line 3305 "/export/home/ffslavex/mozilla-1.9.2-solaris-x86-nightly/build/toolkit/mozapps/update/src/nsUpdateService.js.in"

